@@ -9,7 +9,7 @@ from click.utils import LazyFile
 from schemathesis.checks import CHECKS, load_all_checks
 from schemathesis.cli.commands.run import executor, validation
 from schemathesis.cli.commands.run.filters import with_filters
-from schemathesis.cli.constants import DEFAULT_WORKERS, MAX_WORKERS, MIN_WORKERS
+from schemathesis.cli.constants import MAX_WORKERS, MIN_WORKERS
 from schemathesis.cli.core import ensure_color
 from schemathesis.cli.ext.groups import group, grouped_option
 from schemathesis.cli.ext.options import (
@@ -62,7 +62,7 @@ DEFAULT_PHASES = ["examples", "coverage", "fuzzing", "stateful"]
         ["auto", *list(map(str, range(MIN_WORKERS, MAX_WORKERS + 1)))],
         choices_repr=f"[auto, {MIN_WORKERS}-{MAX_WORKERS}]",
     ),
-    default=str(DEFAULT_WORKERS),
+    default=None,
     show_default=True,
     callback=validation.convert_workers,
     metavar="",
@@ -172,7 +172,7 @@ DEFAULT_PHASES = ["examples", "coverage", "fuzzing", "stateful"]
     help="Skip deprecated operations",
     is_flag=True,
     is_eager=True,
-    default=False,
+    default=None,
     show_default=True,
 )
 @group("Network requests options")
@@ -206,7 +206,7 @@ DEFAULT_PHASES = ["examples", "coverage", "fuzzing", "stateful"]
     "request_tls_verify",
     help="Path to CA bundle for TLS verification, or 'false' to disable",
     type=str,
-    default="true",
+    default=None,
     show_default=True,
     callback=validation.convert_boolean_string,
 )
@@ -280,13 +280,13 @@ DEFAULT_PHASES = ["examples", "coverage", "fuzzing", "stateful"]
     help="Retain exact byte sequence of payloads in cassettes, encoded as base64",
     type=bool,
     is_flag=True,
-    default=False,
+    default=None,
     callback=validation.validate_preserve_bytes,
 )
 @grouped_option(
     "--output-sanitize",
     type=str,
-    default="true",
+    default=None,
     show_default=True,
     help="Enable or disable automatic output sanitization to obscure sensitive data",
     metavar="BOOLEAN",
@@ -296,7 +296,7 @@ DEFAULT_PHASES = ["examples", "coverage", "fuzzing", "stateful"]
     "--output-truncate",
     help="Truncate schemas and responses in error messages",
     type=str,
-    default="true",
+    default=None,
     show_default=True,
     metavar="BOOLEAN",
     callback=validation.convert_boolean_string,
@@ -331,20 +331,21 @@ DEFAULT_PHASES = ["examples", "coverage", "fuzzing", "stateful"]
     "generation_no_shrink",
     help="Disable test case shrinking. Makes test failures harder to debug but improves performance",
     is_flag=True,
+    default=None,
 )
 @grouped_option(
     "--generation-deterministic",
     help="Enables deterministic mode, which eliminates random variation between tests",
     is_flag=True,
     is_eager=True,
-    default=False,
+    default=None,
     show_default=True,
 )
 @grouped_option(
     "--generation-allow-x00",
     help="Whether to allow the generation of 'NULL' bytes within strings",
     type=str,
-    default="true",
+    default=None,
     show_default=True,
     metavar="BOOLEAN",
     callback=validation.convert_boolean_string,
@@ -353,7 +354,7 @@ DEFAULT_PHASES = ["examples", "coverage", "fuzzing", "stateful"]
     "--generation-codec",
     help="The codec used for generating strings",
     type=str,
-    default="utf-8",
+    default=None,
     callback=validation.validate_generation_codec,
 )
 @grouped_option(
@@ -371,7 +372,7 @@ DEFAULT_PHASES = ["examples", "coverage", "fuzzing", "stateful"]
     "--generation-with-security-parameters",
     help="Whether to generate security parameters",
     type=str,
-    default="true",
+    default=None,
     show_default=True,
     callback=validation.convert_boolean_string,
     metavar="BOOLEAN",
@@ -380,7 +381,7 @@ DEFAULT_PHASES = ["examples", "coverage", "fuzzing", "stateful"]
     "--generation-graphql-allow-null",
     help="Whether to use `null` values for optional arguments in GraphQL queries",
     type=str,
-    default="true",
+    default=None,
     show_default=True,
     callback=validation.convert_boolean_string,
     metavar="BOOLEAN",
@@ -398,7 +399,7 @@ DEFAULT_PHASES = ["examples", "coverage", "fuzzing", "stateful"]
     "generation_unique_inputs",
     help="Force the generation of unique test cases",
     is_flag=True,
-    default=False,
+    default=None,
     show_default=True,
     metavar="BOOLEAN",
 )
@@ -440,15 +441,15 @@ def run(
     exclude_operation_id_regex: str | None,
     include_by: Callable | None = None,
     exclude_by: Callable | None = None,
-    exclude_deprecated: bool = False,
-    workers: int = DEFAULT_WORKERS,
+    exclude_deprecated: bool | None = None,
+    workers: int | None = None,
     base_url: str | None,
     wait_for_schema: float | None = None,
     suppress_health_check: list[HealthCheck] | None,
     warnings: bool | list[SchemathesisWarning] | None,
     rate_limit: str | None = None,
     request_timeout: int | None = None,
-    request_tls_verify: bool = True,
+    request_tls_verify: bool | None = None,
     request_cert: str | None = None,
     request_cert_key: str | None = None,
     request_proxy: str | None = None,
@@ -457,21 +458,21 @@ def run(
     report_junit_path: LazyFile | None = None,
     report_vcr_path: LazyFile | None = None,
     report_har_path: LazyFile | None = None,
-    report_preserve_bytes: bool = False,
-    output_sanitize: bool = True,
-    output_truncate: bool = True,
+    report_preserve_bytes: bool | None = None,
+    output_sanitize: bool | None = None,
+    output_truncate: bool | None = None,
     generation_modes: list[GenerationMode],
     generation_seed: int | None = None,
     generation_max_examples: int | None = None,
     generation_maximize: list[MetricFunction] | None,
-    generation_deterministic: bool = False,
+    generation_deterministic: bool | None = None,
     generation_database: str | None = None,
-    generation_unique_inputs: bool = False,
-    generation_allow_x00: bool = True,
-    generation_graphql_allow_null: bool = True,
-    generation_with_security_parameters: bool = True,
-    generation_codec: str = "utf-8",
-    generation_no_shrink: bool = False,
+    generation_unique_inputs: bool | None = None,
+    generation_allow_x00: bool | None = None,
+    generation_graphql_allow_null: bool | None = None,
+    generation_with_security_parameters: bool | None = None,
+    generation_codec: str | None = None,
+    generation_no_shrink: bool | None = None,
     force_color: bool = False,
     no_color: bool = False,
     **__kwargs: Any,
