@@ -98,7 +98,7 @@ class ProjectConfig(DiffBase):
         workers: int | Literal["auto"] = DEFAULT_WORKERS,
         proxy: str | None = None,
         continue_on_failure: bool | None = None,
-        tls_verify: bool | str | None = None,
+        tls_verify: bool | str = True,
         rate_limit: str | None = None,
         request_timeout: float | int | None = None,
         request_cert: str | None = None,
@@ -155,7 +155,7 @@ class ProjectConfig(DiffBase):
             workers=data.get("workers", DEFAULT_WORKERS),
             proxy=resolve(data.get("proxy")),
             continue_on_failure=data.get("continue-on-failure", None),
-            tls_verify=resolve(data.get("tls-verify")),
+            tls_verify=resolve(data.get("tls-verify", True)),
             rate_limit=resolve(data.get("rate-limit")),
             request_timeout=data.get("request-timeout"),
             request_cert=resolve(data.get("request-cert")),
